package sun.awt.image;

import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.Color;
import java.awt.Graphics;
import java.awt.image.ImageObserver;
import java.awt.image.Raster;
import java.awt.image.DataBufferInt;
import java.awt.image.DirectColorModel;
import java.awt.image.IndexColorModel;
import java.util.Hashtable;
import java.awt.image.ColorModel;
import java.awt.image.WritableRaster;
import java.awt.image.BufferedImage;
import java.awt.Rectangle;
import java.awt.image.ImageConsumer;

public class ImageRepresentation extends ImageWatched implements ImageConsumer
{
    InputStreamImageSource src;
    ToolkitImage image;
    int tag;
    long pData;
    int width;
    int height;
    int hints;
    int availinfo;
    Rectangle newbits;
    BufferedImage bimage;
    WritableRaster biRaster;
    protected ColorModel cmodel;
    ColorModel srcModel;
    int[] srcLUT;
    int srcLUTtransIndex;
    int numSrcLUT;
    boolean forceCMhint;
    int sstride;
    boolean isDefaultBI;
    boolean isSameCM;
    static boolean s_useNative;
    private boolean consuming;
    private int numWaiters;
    
    private static native void initIDs();
    
    public ImageRepresentation(ToolkitImage image, ColorModel colorModel, boolean forceCMhint) {
        this.width = -1;
        this.height = -1;
        this.srcModel = null;
        this.srcLUT = null;
        this.srcLUTtransIndex = -1;
        this.numSrcLUT = 0;
        this.isDefaultBI = false;
        this.isSameCM = false;
        this.consuming = false;
        this.image = image;
        if ((this.image.getSource() instanceof InputStreamImageSource)) {
            this.src = (InputStreamImageSource)this.image.getSource();
        }
        this.setColorModel(colorModel);
        this.forceCMhint = forceCMhint;
    }
    
    public synchronized void reconstruct(int n) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        int n2 = n & ~this.availinfo;
        if ((this.availinfo & 0x40) == 0x0 && n2 != 0) {
            ++this.numWaiters;
            try {
                this.startProduction();
                for (int n3 = n & ~this.availinfo; (this.availinfo & 0x40) == 0x0 && n3 != 0; n3 = (n & ~this.availinfo)) {
                    try {
                        this.wait();
                    }
                    catch (InterruptedException ex) {
                        Thread.currentThread().interrupt();
                        return;
                    }
                }
            }
            finally {
                this.decrementWaiters();
            }
        }
    }
    
    @Override
    public void setDimensions(int width, int height) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        this.image.setDimensions(width, height);
        this.newInfo(this.image, 3, 0, 0, width, height);
        if (width <= 0 || height <= 0) {
            this.imageComplete(1);
            return;
        }
        if (this.width != width || this.height != height) {
            this.bimage = null;
        }
        this.width = width;
        this.height = height;
        this.availinfo |= 0x3;
        return;
    }
    
    public int getWidth() {
        return this.width;
    }
    
    public int getHeight() {
        return this.height;
    }
    
    ColorModel getColorModel() {
        return this.cmodel;
    }
    
    BufferedImage getBufferedImage() {
        return this.bimage;
    }
    
    protected BufferedImage createImage(ColorModel cm, WritableRaster raster, boolean isRasterPremultiplied, Hashtable hashtable) {
        BufferedImage bufferedImage = new BufferedImage(cm, raster, isRasterPremultiplied, null);
        bufferedImage.setAccelerationPriority(this.image.getAccelerationPriority());
        return bufferedImage;
    }
    
    @Override
    public void setProperties(Hashtable<?, ?> properties) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        this.image.setProperties(properties);
        this.newInfo(this.image, 4, 0, 0, 0, 0);
    }
    
    @Override
    public void setColorModel(ColorModel colorModel) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        this.srcModel = colorModel;
        if ((colorModel instanceof IndexColorModel)) {
            if (colorModel.getTransparency() == 3) {
                this.cmodel = ColorModel.getRGBdefault();
                this.srcLUT = null;
            }
            else {
                IndexColorModel indexColorModel = (IndexColorModel)colorModel;
                this.numSrcLUT = indexColorModel.getMapSize();
                indexColorModel.getRGBs(this.srcLUT = new int[Math.max(this.numSrcLUT, 256)]);
                this.srcLUTtransIndex = indexColorModel.getTransparentPixel();
                this.cmodel = colorModel;
            }
        }
        else if (this.cmodel == null) {
            this.cmodel = colorModel;
            this.srcLUT = null;
        }
        else if ((colorModel instanceof DirectColorModel)) {
            DirectColorModel directColorModel = (DirectColorModel)colorModel;
            if (directColorModel.getRedMask() == 16711680 && directColorModel.getGreenMask() == 65280 && directColorModel.getBlueMask() == 255) {
                this.cmodel = colorModel;
                this.srcLUT = null;
            }
        }
        this.isSameCM = (this.cmodel == colorModel);
    }
    
    void createBufferedImage() {
        this.isDefaultBI = false;
        try {
            this.biRaster = this.cmodel.createCompatibleWritableRaster(this.width, this.height);
            this.bimage = this.createImage(this.cmodel, this.biRaster, this.cmodel.isAlphaPremultiplied(), null);
        }
        catch (Exception ex) {
            this.cmodel = ColorModel.getRGBdefault();
            this.biRaster = this.cmodel.createCompatibleWritableRaster(this.width, this.height);
            this.bimage = this.createImage(this.cmodel, this.biRaster, false, null);
        }
        int type = this.bimage.getType();
        if (this.cmodel == ColorModel.getRGBdefault() || type == 1 || type == 3) {
            this.isDefaultBI = true;
        }
        else if ((this.cmodel instanceof DirectColorModel)) {
            DirectColorModel directColorModel = (DirectColorModel)this.cmodel;
            if (directColorModel.getRedMask() == 16711680 && directColorModel.getGreenMask() == 65280 && directColorModel.getBlueMask() == 255) {
                this.isDefaultBI = true;
            }
        }
    }
    
    private void convertToRGB() {
        int width = this.bimage.getWidth();
        int height = this.bimage.getHeight();
        int size = width * height;
        DataBufferInt dataBuffer = new DataBufferInt(size);
        int[] stealData = SunWritableRaster.stealData(dataBuffer, 0);
        if (this.cmodel instanceof IndexColorModel && this.biRaster instanceof ByteComponentRaster && this.biRaster.getNumDataElements() == 1) {
            ByteComponentRaster byteComponentRaster = (ByteComponentRaster)this.biRaster;
            byte[] dataStorage = byteComponentRaster.getDataStorage();
            int dataOffset = byteComponentRaster.getDataOffset(0);
            for (int k = 0; k < size; ++k) {
                stealData[k] = this.srcLUT[dataStorage[dataOffset + k] & 0xFF];
            }
        }
        else {
            Object dataElements = null;
            int n = 0;
            for (int i = 0; i < height; ++i) {
                for (int j = 0; j < width; ++j) {
                    dataElements = this.biRaster.getDataElements(j, i, dataElements);
                    stealData[n++] = this.cmodel.getRGB(dataElements);
                }
            }
        }
        SunWritableRaster.markDirty(dataBuffer);
        this.isSameCM = false;
        this.cmodel = ColorModel.getRGBdefault();
        this.biRaster = Raster.createPackedRaster(dataBuffer, width, height, width, new int[] { 16711680, 65280, 255, -16777216 }, null);
        this.bimage = this.createImage(this.cmodel, this.biRaster, this.cmodel.isAlphaPremultiplied(), null);
        this.srcLUT = null;
        this.isDefaultBI = true;
    }
    
    @Override
    public void setHints(int hints) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        this.hints = hints;
    }
    
    private native boolean setICMpixels(int p0, int p1, int p2, int p3, int[] p4, byte[] p5, int p6, int p7, IntegerComponentRaster p8);
    
    private native boolean setDiffICM(int p0, int p1, int p2, int p3, int[] p4, int p5, int p6, IndexColorModel p7, byte[] p8, int p9, int p10, ByteComponentRaster p11, int p12);
    
    @Override
    public void setPixels(int i, int j, int n, int n2, ColorModel colorModel, byte[] array, int n3, int n4) {
        int n5 = n3;
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        synchronized (this) {
            if (this.bimage == null) {
                if (this.cmodel == null) {
                    this.cmodel = colorModel;
                }
                this.createBufferedImage();
            }
            if (n <= 0 || n2 <= 0) {
                return;
            }
            int width = this.biRaster.getWidth();
            int height = this.biRaster.getHeight();
            width = i + n;
            height = j + n2;
            if (i < 0) {
                n3 -= i;
                i = 0;
            }
            else if (width < 0) {
            }
            if (j < 0) {
                n3 -= j * n4;
                j = 0;
            }
            else if (height < 0) {
            }
            if (width > width) {
            }
            if (height > height) {
            }
            if (i >= width || j >= height) {
                return;
            }
            n = width - i;
            n2 = height - j;
            if (n3 < 0 || n3 >= array.length) {
                throw new ArrayIndexOutOfBoundsException("Data offset out of bounds.");
            }
            int n6 = array.length - n3;
            if (n6 < n) {
                throw new ArrayIndexOutOfBoundsException("Data array is too short.");
            }
            int n7;
            if (n4 < 0) {
                n7 = n3 / -n4 + 1;
            }
            else if (n4 > 0) {
                n7 = (n6 - n) / n4 + 1;
            }
            else {
                n7 = n2;
            }
            if (n2 > n7) {
                throw new ArrayIndexOutOfBoundsException("Data array is too short.");
            }
            if (this.isSameCM && this.cmodel != colorModel && this.srcLUT != null && colorModel instanceof IndexColorModel && this.biRaster instanceof ByteComponentRaster) {
                IndexColorModel indexColorModel = (IndexColorModel)colorModel;
                ByteComponentRaster byteComponentRaster = (ByteComponentRaster)this.biRaster;
                int numSrcLUT = this.numSrcLUT;
                if (!this.setDiffICM(i, j, n, n2, this.srcLUT, this.srcLUTtransIndex, this.numSrcLUT, indexColorModel, array, n3, n4, byteComponentRaster, byteComponentRaster.getDataOffset(0))) {
                    this.convertToRGB();
                }
                else {
                    byteComponentRaster.markDirty();
                    if (numSrcLUT != this.numSrcLUT) {
                        int hasAlpha = indexColorModel.hasAlpha();
                        if (this.srcLUTtransIndex != -1) {
                            hasAlpha = true;
                        }
                        int pixelSize = indexColorModel.getPixelSize();
                        IndexColorModel cmodel = new IndexColorModel(pixelSize, this.numSrcLUT, this.srcLUT, 0, (boolean)hasAlpha, this.srcLUTtransIndex, (pixelSize > 8) ? 1 : 0);
                        this.cmodel = cmodel;
                        this.bimage = this.createImage(cmodel, byteComponentRaster, false, null);
                    }
                    return;
                }
            }
            if (this.isDefaultBI) {
                IntegerComponentRaster integerComponentRaster = (IntegerComponentRaster)this.biRaster;
                if (this.srcLUT != null && colorModel instanceof IndexColorModel) {
                    if (colorModel != this.srcModel) {
                        ((IndexColorModel)colorModel).getRGBs(this.srcLUT);
                        this.srcModel = colorModel;
                    }
                    if (ImageRepresentation.s_useNative) {
                        if (this.setICMpixels(i, j, n, n2, this.srcLUT, array, n3, n4, integerComponentRaster)) {
                            integerComponentRaster.markDirty();
                        }
                        else {
                            this.abort();
                            return;
                        }
                    }
                    else {
                        int[] array4 = new int[n * n2];
                        int n8 = 0;
                        for (int l = 0; l < n2; ++l, n5 += n4) {
                            for (int n9 = 0; n9 < n; ++n9) {
                                array4[n8++] = this.srcLUT[array[n5++] & 0xFF];
                            }
                        }
                        integerComponentRaster.setDataElements(i, j, n, n2, array4);
                    }
                }
                else {
                    int[] array3 = new int[n];
                    int k;
                    for (; j < j + n2; ++j, n5 += n4) {
                        for (k = 0; k < n; ++k) {
                            array3[k] = colorModel.getRGB(array[n5++] & 0xFF);
                        }
                        integerComponentRaster.setDataElements(i, j, n, 1, array3);
                    }
                    this.availinfo |= 0x8;
                }
            }
            else if (this.cmodel == colorModel && this.biRaster instanceof ByteComponentRaster && this.biRaster.getNumDataElements() == 1) {
                ByteComponentRaster byteComponentRaster2 = (ByteComponentRaster)this.biRaster;
                if (n3 == 0 && n4 == n) {
                    byteComponentRaster2.putByteData(i, j, n, n2, array);
                }
                else {
                    byte[] array2 = new byte[n];
                    for (, ; j < j + n2; n3 += n4, ++j) {
                        System.arraycopy(array, n3, array2, 0, n);
                        byteComponentRaster2.putByteData(i, j, n, 1, array2);
                    }
                }
            }
            else {
                for (; j < j + n2; ++j, n5 += n4) {
                    for (; i < i + n; ++i) {
                        this.bimage.setRGB(i, j, colorModel.getRGB(array[n5++] & 0xFF));
                    }
                }
                this.availinfo |= 0x8;
            }
        }
        if ((this.availinfo & 0x10) == 0x0) {
            this.newInfo(this.image, 8, i, j, n, n2);
        }
    }
    
    @Override
    public void setPixels(int i, int j, int n, int n2, ColorModel colorModel, int[] array, int n3, int n4) {
        int n5 = n3;
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        synchronized (this) {
            if (this.bimage == null) {
                if (this.cmodel == null) {
                    this.cmodel = colorModel;
                }
                this.createBufferedImage();
            }
            int[] array2 = new int[n];
            if ((this.cmodel instanceof IndexColorModel)) {
                this.convertToRGB();
            }
            if (colorModel == this.cmodel && this.biRaster instanceof IntegerComponentRaster) {
                IntegerComponentRaster integerComponentRaster2 = (IntegerComponentRaster)this.biRaster;
                if (n3 == 0 && n4 == n) {
                    integerComponentRaster2.setDataElements(i, j, n, n2, array);
                }
                else {
                    for (; j < j + n2; ++j, n5 += n4) {
                        System.arraycopy(array, n5, array2, 0, n);
                        integerComponentRaster2.setDataElements(i, j, n, 1, array2);
                    }
                }
            }
            else {
                if (colorModel.getTransparency() != 1) {
                    int transparency = this.cmodel.getTransparency();
                    ColorModel cmodel = this.cmodel;
                    if (transparency == 1) {
                        this.convertToRGB();
                    }
                }
                if (this.isDefaultBI) {
                    IntegerComponentRaster integerComponentRaster = (IntegerComponentRaster)this.biRaster;
                    int[] dataStorage = integerComponentRaster.getDataStorage();
                    if (this.cmodel.equals(colorModel)) {
                        for (int scanlineStride = integerComponentRaster.getScanlineStride(), n6 = j * scanlineStride + i, l = 0; l < n2; n6 += scanlineStride, ++l, n5 += n4) {
                            System.arraycopy(array, n5, dataStorage, n6, n);
                        }
                        integerComponentRaster.markDirty();
                    }
                    else {
                        int k;
                        for (; j < j + n2; ++j, n5 += n4) {
                            for (k = 0; k < n; ++k) {
                                array2[k] = colorModel.getRGB(array[n5++]);
                            }
                            integerComponentRaster.setDataElements(i, j, n, 1, array2);
                        }
                    }
                    this.availinfo |= 0x8;
                }
                else {
                    Object dataElements = null;
                    for (; j < j + n2; ++j, n5 += n4) {
                        for (; i < i + n; ++i) {
                            dataElements = this.cmodel.getDataElements(colorModel.getRGB(array[n5++]), dataElements);
                            this.biRaster.setDataElements(i, j, dataElements);
                        }
                    }
                    this.availinfo |= 0x8;
                }
            }
        }
        if ((this.availinfo & 0x10) == 0x0) {
            this.newInfo(this.image, 8, i, j, n, n2);
        }
    }
    
    public BufferedImage getOpaqueRGBImage() {
        if (this.bimage.getType() == 2) {
            int width = this.bimage.getWidth();
            int height = this.bimage.getHeight();
            int n = width * height;
            DataBufferInt dataBuffer = (DataBufferInt)this.biRaster.getDataBuffer();
            int[] stealData = SunWritableRaster.stealData(dataBuffer, 0);
            int i = 0;
            while (i < n) {
                if (stealData[i] >>> 24 != 255) {
                    return this.bimage;
                }
                ++i;
            }
            DirectColorModel directColorModel = new DirectColorModel(24, 16711680, 65280, 255);
            WritableRaster packedRaster = Raster.createPackedRaster(dataBuffer, width, height, width, new int[] { 16711680, 65280, 255 }, null);
            try {
                return this.createImage(directColorModel, packedRaster, false, null);
            }
            catch (Exception ex) {
                return this.bimage;
            }
        }
        else {
            return this.bimage;
        }
    }
    
    @Override
    public void imageComplete(int n) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        int n2;
        int n3;
        switch (n) {
            case 1: {
                this.image.addInfo(64);
                n2 = true;
                n3 = 64;
                this.dispose();
                break;
            }
            case 3: {
                n2 = true;
                n3 = 32;
                break;
            }
            case 2: {
                n2 = false;
                n3 = 16;
                break;
            }
            default: {
                n2 = true;
                n3 = 128;
                break;
            }
        }
        synchronized (this) {
            if (!(n2 == 0)) {
                this.image.getSource().removeConsumer(this);
                this.consuming = false;
                this.newbits = null;
                if (this.bimage != null) {
                    this.bimage = this.getOpaqueRGBImage();
                }
            }
            this.availinfo |= n3;
            this.notifyAll();
        }
        this.newInfo(this.image, n3, 0, 0, this.width, this.height);
        this.image.infoDone(n);
    }
    
    void startProduction() {
        if (!this.consuming) {
            this.consuming = true;
            this.image.getSource().startProduction(this);
        }
    }
    
    private synchronized void checkConsumption() {
        if (this.isWatcherListEmpty() && this.numWaiters == 0 && (this.availinfo & 0x20) == 0x0) {
            this.dispose();
        }
    }
    
    public synchronized void notifyWatcherListEmpty() {
        this.checkConsumption();
    }
    
    private synchronized void decrementWaiters() {
        --this.numWaiters;
        this.checkConsumption();
    }
    
    public boolean prepare(ImageObserver imageObserver) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        if ((this.availinfo & 0x40) != 0x0) {
            if (imageObserver != null) {
                imageObserver.imageUpdate(this.image, 192, -1, -1, -1, -1);
            }
            return false;
        }
        else {
            int n = (this.availinfo & 0x20) != 0x0;
            if (n == 0) {
                this.addWatcher(imageObserver);
                this.startProduction();
                n = ((this.availinfo & 0x20) != 0x0);
            }
            return n;
        }
    }
    
    public int check(ImageObserver imageObserver) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        if ((this.availinfo & 0x60) == 0x0) {
            this.addWatcher(imageObserver);
        }
        return this.availinfo;
    }
    
    public boolean drawToBufImage(Graphics graphics, ToolkitImage toolkitImage, int n, int n2, Color color, ImageObserver imageObserver) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        if ((this.availinfo & 0x40) != 0x0) {
            if (imageObserver != null) {
                imageObserver.imageUpdate(this.image, 192, -1, -1, -1, -1);
            }
            return false;
        }
        else {
            int n3 = (this.availinfo & 0x20) != 0x0;
            int n4 = (this.availinfo & 0x80) != 0x0;
            if (n3 == 0 && n4 == 0) {
                this.addWatcher(imageObserver);
                this.startProduction();
                n3 = ((this.availinfo & 0x20) != 0x0);
            }
            if (n3 || 0x0 != (this.availinfo & 0x10)) {
                graphics.drawImage(this.bimage, n, n2, color, null);
            }
            return n3;
        }
    }
    
    public boolean drawToBufImage(Graphics graphics, ToolkitImage toolkitImage, int n, int n2, int n3, int n4, Color color, ImageObserver imageObserver) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        if ((this.availinfo & 0x40) != 0x0) {
            if (imageObserver != null) {
                imageObserver.imageUpdate(this.image, 192, -1, -1, -1, -1);
            }
            return false;
        }
        else {
            int n5 = (this.availinfo & 0x20) != 0x0;
            int n6 = (this.availinfo & 0x80) != 0x0;
            if (n5 == 0 && n6 == 0) {
                this.addWatcher(imageObserver);
                this.startProduction();
                n5 = ((this.availinfo & 0x20) != 0x0);
            }
            if (n5 || 0x0 != (this.availinfo & 0x10)) {
                graphics.drawImage(this.bimage, n, n2, n3, n4, color, null);
            }
            return n5;
        }
    }
    
    public boolean drawToBufImage(Graphics graphics, ToolkitImage toolkitImage, int n, int n2, int n3, int n4, int n5, int n6, int n7, int n8, Color color, ImageObserver imageObserver) {
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        if ((this.availinfo & 0x40) != 0x0) {
            if (imageObserver != null) {
                imageObserver.imageUpdate(this.image, 192, -1, -1, -1, -1);
            }
            return false;
        }
        else {
            int n9 = (this.availinfo & 0x20) != 0x0;
            int n10 = (this.availinfo & 0x80) != 0x0;
            if (n9 == 0 && n10 == 0) {
                this.addWatcher(imageObserver);
                this.startProduction();
                n9 = ((this.availinfo & 0x20) != 0x0);
            }
            if (n9 || 0x0 != (this.availinfo & 0x10)) {
                graphics.drawImage(this.bimage, n, n2, n3, n4, n5, n6, n7, n8, color, null);
            }
            return n9;
        }
    }
    
    public boolean drawToBufImage(Graphics graphics, ToolkitImage toolkitImage, AffineTransform affineTransform, ImageObserver imageObserver) {
        Graphics2D graphics2D = (Graphics2D)graphics;
        if (this.src != null) {
            this.src.checkSecurity(null, false);
        }
        if ((this.availinfo & 0x40) != 0x0) {
            if (imageObserver != null) {
                imageObserver.imageUpdate(this.image, 192, -1, -1, -1, -1);
            }
            return false;
        }
        else {
            int n = (this.availinfo & 0x20) != 0x0;
            int n2 = (this.availinfo & 0x80) != 0x0;
            if (n == 0 && n2 == 0) {
                this.addWatcher(imageObserver);
                this.startProduction();
                n = ((this.availinfo & 0x20) != 0x0);
            }
            if (n || 0x0 != (this.availinfo & 0x10)) {
                graphics2D.drawImage(this.bimage, affineTransform, null);
            }
            return n;
        }
    }
    
    synchronized void abort() {
        this.image.getSource().removeConsumer(this);
        this.consuming = false;
        this.newbits = null;
        this.bimage = null;
        this.biRaster = null;
        this.cmodel = null;
        this.srcLUT = null;
        this.isDefaultBI = false;
        this.isSameCM = false;
        this.newInfo(this.image, 128, -1, -1, -1, -1);
        this.availinfo &= 0xFFFFFF87;
    }
    
    synchronized void dispose() {
        this.image.getSource().removeConsumer(this);
        this.consuming = false;
        this.newbits = null;
        this.availinfo &= 0xFFFFFFC7;
    }
    
    public void setAccelerationPriority(float accelerationPriority) {
        if (this.bimage != null) {
            this.bimage.setAccelerationPriority(accelerationPriority);
        }
    }
    
    static {
        NativeLibLoader.loadLibraries();
        initIDs();
        ImageRepresentation.s_useNative = true;
    }
}
